Encounter = class(Turbine.UI.Control)
function Encounter:Constructor()
    Turbine.UI.Control.Constructor(self)

    self.MoralePower = {
        ["Morale"] = 1,
        ["Power"] = 2,
        ["Wrath"] = 3, --?
    }

    self.CombatStartTime = Turbine.Engine.GetGameTime()
    self.oldGameTime = Turbine.Engine.GetGameTime()

    self:SetWantsUpdates(true)

    self.UpdateDelay = 0.1
    self.decimalPlaces = 1
    --self.StartTimesLUT[self.CombatStartTime] = Turbine.Engine.GetGameTime()

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()

    -- Per seconds
    self.DPSMapped = {}

    -- Skill Data
    self.SkillsMapped = {}
    self.ItemsMapped = {}
    self.SkillIcons = {}
    self.SkillDamage = {}
    self.SkillHits = {}
    self.SkillDamageTypes = {}
    self.SkillMaxDamage = {}
    self.SkillTotalHits = {}

    -- Target Skill Data
    self.TargetSkillsMapped = {}
    self.TargetItemsMapped = {}
    self.TargetSkillIcons = {}
    self.TargetSkillDamage = {}
    self.TargetSkillHits = {}
    self.TargetSkillDamageTypes = {}
    self.TargetSkillMaxDamage = {}
    self.TargetSkillTotalHits = {}

    -- Target Data
    self.TargetDamage = {}
    self.TargetHits = {}
    self.TargetDamageTypes = {}

    -- Misc tracking
    self.TargetBreakdown = {}
    self.PowerMapped = {}
    self.MoraleMapped = {}
    self.DamageMapped = {}

    -- Damage types totals
    self.DamagetTypeData = {
        -- True damage - reflect 
        [""] = 0,

        -- Physical damages
        ["Common"] = 0,
        ["Ancient Dwarf-make"] = 0,
        ["Beleriand"] = 0,
        ["Fell-wrought"] = 0,
        ["Orc-craft"] = 0,
        ["Westernesse"] = 0,

        -- Tactical types
        ["Acid"] = 0,
        ["Fire"] = 0,
        ["Frost"] = 0,
        ["Light"] = 0,
        ["Lightning"] = 0,
        ["Shadow"] = 0,
    }

    self.HitTypeData = {
        [""] = 0,
        ["critical "] = 0,
        ["devastating "] = 0,
        ["partially blocked "] = 0,
        ["partially parried "] = 0,
        ["partially evaded "] = 0,
    }

    -- Totals
    self.CurrentTotalDamage = 0
    self.Actions = 0
    self.ItemActions = 0
    self.MaxDamage = 0
    self.TotalHits = 0

end

function Encounter:UpdateDamage(damage, damageType, hitType, skillName, target)

    -- Update total
    self.CurrentTotalDamage = self.CurrentTotalDamage + damage

    -- Add to type totals
    self.DamagetTypeData[damageType] = self.DamagetTypeData[damageType] + damage

    -- Add to type totals
    self.TotalHits = self.TotalHits + 1
    self.HitTypeData[hitType] = self.HitTypeData[hitType] + 1

    -- Max damage
    if damage > self.MaxDamage then self.MaxDamage = damage end

    -- Skill damage saving
    self:AddSkillDamage(damage, skillName, damageType, hitType)

    -- Target damage saving
    self:AddTargetDamage(damage, skillName, damageType, hitType, target)

end

function Encounter:AddTargetDamage(damage, skillName, damageType, hitType, targetName)
    if not(self.TargetDamage[targetName]) then
        self.TargetDamage[targetName] = 0
        self.TargetHits[targetName] = {
            [""] = 0,
            ["critical "] = 0,
            ["devastating "] = 0,
            ["partially blocked "] = 0,
            ["partially parried "] = 0,
            ["partially evaded "] = 0,
        }
        self.TargetDamageTypes[targetName] = {
            -- True damage - reflect 
            [""] = 0,

            -- Physical damages
            ["Common"] = 0,
            ["Ancient Dwarf-make"] = 0,
            ["Beleriand"] = 0,
            ["Fell-wrought"] = 0,
            ["Orc-craft"] = 0,
            ["Westernesse"] = 0,

            -- Tactical types
            ["Acid"] = 0,
            ["Fire"] = 0,
            ["Frost"] = 0,
            ["Light"] = 0,
            ["Lightning"] = 0,
            ["Shadow"] = 0,
        }
        self.TargetSkillDamage[targetName] = {}
        self.TargetSkillHits[targetName]= {}
        self.TargetSkillDamageTypes[targetName] = {}
    end

    self.TargetDamage[targetName] = self.TargetDamage[targetName] + damage
    self.TargetHits[targetName][hitType] = self.TargetHits[targetName][hitType] + 1
    self.TargetDamageTypes[targetName][damageType] = self.TargetDamageTypes[targetName][damageType] + damage

    self:AddTargetSkillDamage(damage, skillName, damageType, hitType, targetName)
end

function Encounter:AddSkillDamage(damage, skillName, damageType, hitType)
    if not(self.SkillDamage[skillName]) then
        self.SkillDamage[skillName] = 0
        self.SkillHits[skillName] = {
            [""] = 0,
            ["critical "] = 0,
            ["devastating "] = 0,
            ["partially blocked "] = 0,
            ["partially parried "] = 0,
            ["partially evaded "] = 0,
        }
        self.SkillDamageTypes[skillName] = {
            -- True damage - reflect 
            [""] = 0,

            -- Physical damages
            ["Common"] = 0,
            ["Ancient Dwarf-make"] = 0,
            ["Beleriand"] = 0,
            ["Fell-wrought"] = 0,
            ["Orc-craft"] = 0,
            ["Westernesse"] = 0,

            -- Tactical types
            ["Acid"] = 0,
            ["Fire"] = 0,
            ["Frost"] = 0,
            ["Light"] = 0,
            ["Lightning"] = 0,
            ["Shadow"] = 0,
        }
        self.SkillMaxDamage[skillName] = 0
        self.SkillTotalHits[skillName] = 0
    end
    if damage > self.SkillMaxDamage[skillName] then self.SkillMaxDamage[skillName] = damage end
    self.SkillTotalHits[skillName] = self.SkillTotalHits[skillName] + 1
    self.SkillDamage[skillName] = self.SkillDamage[skillName] + damage
    self.SkillHits[skillName][hitType] = self.SkillHits[skillName][hitType] + 1
    self.SkillDamageTypes[skillName][damageType] = self.SkillDamageTypes[skillName][damageType] + damage
end

function Encounter:AddTargetSkillDamage(damage, skillName, damageType, hitType, targetName)
    if not(self.TargetSkillDamage[targetName][skillName]) then
        self.TargetSkillDamage[targetName][skillName] = 0
        self.TargetSkillHits[targetName][skillName] = {
            [""] = 0,
            ["critical "] = 0,
            ["devastating "] = 0,
            ["partially blocked "] = 0,
            ["partially parried "] = 0,
            ["partially evaded "] = 0,
        }
        self.TargetSkillDamageTypes[targetName][skillName] = {
            -- True damage - reflect 
            [""] = 0,

            -- Physical damages
            ["Common"] = 0,
            ["Ancient Dwarf-make"] = 0,
            ["Beleriand"] = 0,
            ["Fell-wrought"] = 0,
            ["Orc-craft"] = 0,
            ["Westernesse"] = 0,

            -- Tactical types
            ["Acid"] = 0,
            ["Fire"] = 0,
            ["Frost"] = 0,
            ["Light"] = 0,
            ["Lightning"] = 0,
            ["Shadow"] = 0,
        }
    end

    self.TargetSkillDamage[targetName][skillName] = self.TargetSkillDamage[targetName][skillName] + damage
    self.TargetSkillHits[targetName][skillName][hitType] = self.TargetSkillHits[targetName][skillName][hitType] + 1
    self.TargetSkillDamageTypes[targetName][skillName][damageType] = self.TargetSkillDamageTypes[targetName][skillName][damageType] + damage
end

function Encounter:GetTargetInfo()
    return self.TargetDamage, self.TargetHits, self.TargetDamageTypes
end

function Encounter:GetSkillInfo()
    return self.SkillDamage, self.SkillHits, self.SkillDamageTypes
end

function Encounter:Update()
    local currentGameTime = Turbine.Engine.GetGameTime()
    if currentGameTime == self.oldGameTime then
        return(true)
    end
    local delta = currentGameTime - self.oldGameTime
    if delta < self.UpdateDelay then return end

    self.oldGameTime =  numberRound(currentGameTime, self.decimalPlaces)

    if self.player:IsInCombat() then
        --self.CurrentItem = self.CurrentItem + 1

        self.DPSMapped[self.oldGameTime] = self.CurrentTotalDamage / (Turbine.Engine.GetGameTime() - self.CombatStartTime)
        self.PowerMapped[self.oldGameTime] = self.player:GetPower()
        self.MoraleMapped[self.oldGameTime] = self.player:GetMorale()
        self.DamageMapped[self.oldGameTime] = self.CurrentTotalDamage
        --self.AdddamageContainerItem(self.DPSMapped[self.CurrentTimeStamp], currentGameTime)

        --smallPP:SetDPS(self.DPSMapped[self.oldGameTime])
        --ChatOutput(self.CurrentTotalDamage / (Turbine.Engine.GetGameTime() - self.CombatStartTime) / 50000 * 200)
    end
end

function Encounter:GetTotalDamage()
    return self.CurrentTotalDamage
end

function Encounter:GetTotalCrit()
    return self.HitTypeData["critical "]
end

function Encounter:GetTotalCritPercentage()
    return self.HitTypeData["critical "] / self.TotalHits * 100
end

function Encounter:GetDPS()
    return self.DPSMapped[self.oldGameTime]
end

function Encounter:GetInformation()
    return {
        ["DPSMapped"] = self.DPSMapped,

        -- Skill Data
        ["SkillsMapped"] = self.SkillsMapped,
        ["ItemsMapped"] = self.ItemsMapped,
        ["SkillIcons"] = self.SkillIcons,
        ["SkillDamage"] = self.SkillDamage,
        ["SkillHits"] = self.SkillHits,
        ["SkillDamageTypes"] = self.SkillDamageTypes,
        ["SkillMaxDamage"] = self.SkillMaxDamage,
        ["SkillTotalHits"] = self.SkillTotalHits,

        -- Target Skill Data
        ["TargetSkillsMapped"] = self.TargetSkillsMapped,
        ["TargetItemsMapped"] = self.TargetItemsMapped,
        ["TargetSkillIcons"] = self.TargetSkillIcons,
        ["TargetSkillDamage"] = self.TargetSkillDamage,
        ["TargetSkillHits"] = self.TargetSkillHits,
        ["TargetSkillDamageTypes"] = self.TargetSkillDamageTypes,
        ["TargetSkillMaxDamage"] = self.SkillMaxDamage,
        ["TargetSkillTotalHits"] = self.SkillTotalHits,
    
        -- Target Data
        ["TargetDamage"] = self.TargetDamage,
        ["TargetHits"] = self.TargetHits,
        ["TargetDamageTypes"] = self.TargetDamageTypes,
    
        -- Misc tracking
        ["TargetBreakdown"] = self.TargetBreakdown,
        ["PowerMapped"] = self.PowerMapped,
        ["MoraleMapped"] = self.MoraleMapped,
        ["DamageMapped"] = self.DamageMapped,
    
        -- Damage types totals
        ["DamagetTypeData"] = self.DamagetTypeData,
    
        ["HitTypeData"] = self.HitTypeData,
    
        -- Totals
        ["CurrentTotalDamage"] = self.CurrentTotalDamage,
        ["Actions"] = self.Actions,
        ["ItemActions"] = self.ItemActions,
        ["MaxDamage"] = self.MaxDamage,
        ["TotalHits"] = self.TotalHits,
    }
end