PlayerEffectEvents = class(Turbine.UI.Control)
function PlayerEffectEvents:Constructor()
  Turbine.UI.Control.Constructor(self)
  self.Player = Turbine.Gameplay.LocalPlayer.GetInstance()
  self.PlayerEffects = self.Player:GetEffects()

  self.callbacks = {}
  self.addedCallbacks = {}

  self.EffectUpdate = function(sender, event, args)
    if type(event)=="function" then
        event(args)
    else
        if type(event)=="table" then
            local size = table.getn(event)
            local i
            for i=1,size do
                if type(event[i])=="function" then
                    event[i](args)
                end
            end
        end
    end
  end

  AddCallback(self.PlayerEffects, "EffectAdded", function(sender, args)
    self.EffectUpdate(self, self.EffectAdded, self.PlayerEffects:Get(args.Index))
  end)

  AddCallback(self.PlayerEffects, "EffectRemoved", function(sender, args)
    if args.Effect then
      self.EffectUpdate(self, self.EffectRemoved, args.Effect)
    end
  end)
end

function PlayerEffectEvents:RemoveCallbacks()
  for key, value in pairs(self.addedCallbacks) do
    RemoveCallback(PlayerEffectsHandler, "EffectAdded", value)
  end
end

GroupEffectEvents = class(Turbine.UI.Control)
function GroupEffectEvents:Constructor()
  Turbine.UI.Control.Constructor(self)
  self.Player = Turbine.Gameplay.LocalPlayer.GetInstance()

  -- Information tables
  self.Group = {}
  self.GroupNameLUT = {}
  self.GroupRats = {}

  self.GroupSize = 0
  self.callbacks = {}
  self.addedCallbacks = {}

  self.EffectUpdate = function(sender, event, args)
    if type(event)=="function" then
        event(args)
    else
        if type(event)=="table" then
            local size = table.getn(event)
            local i
            for i=1,size do
                if type(event[i])=="function" then
                    event[i](args)
                end
            end
        end
    end
  end

  -- Set Group Rat Infor
  self.SetGroupRats = function()
    self.GroupRats = {}
    if Party and Party:GetAssistTargetCount() > 0 then
      for i = 1, Party:GetAssistTargetCount(), 1 do
        self.GroupRats[Party:GetAssistTarget(i):GetName()] = true
      end
    end
  end

  -- Set Group Info
  self.SetGroupInformation = function()
    self.Group = {}
    self.GroupNameLUT = {}
    self.GroupRats = nil
    if Party then
      self.GroupSize = Party:GetMemberCount()
      for i = 1, GroupSize, 1 do
        self.Group[i] = {}
        self.Group[i].player = Party:GetMember(i)
        self.Group[i].name = Party:GetMember(i):GetName()
        self.Group[i].effects = self.Group[i].player:GetEffects()
        self.Group[i].currentEffects = {}
        self.GroupNameLUT[Party:GetMember(i):GetName()] = true
      end
    else
      self.Group[1] = {}
      self.Group[1].player = Player
      self.Group[1].name = Player:GetName()
      self.Group[1].effects = self.Group[1].player:GetEffects()
      self.Group[1].currentEffects = {}
    end

    self.SetGroupRats()
  end

  -- Set Group Triggers
  self.SetGroupReloads = function()
    self.Player.PartyChanged = function()
      ReloadPlugin()
      --ChatOutput("Party changed")
      --if ratTracker ~=  nil then ratTracker:UpdateText() end
    end
    self.Player.RaidChanged = function()
      ReloadPlugin()
      --ChatOutput("Raid changed")
      --if ratTracker ~=  nil then ratTracker:UpdateText() end
    end
    if Party ~= nil then
      Party.MemberAdded = function()
        ReloadPlugin()
        --ChatOutput("Member added")
        --if ratTracker ~=  nil then ratTracker:UpdateText() end
      end
      Party.MemberRemoved = function()
        ReloadPlugin()
        --ChatOutput("Member removed")
        --if ratTracker ~=  nil then ratTracker:UpdateText() end
      end
      Party.LeaderChanged = function()
        ReloadPlugin()
        --if ratTracker ~=  nil then ratTracker:UpdateText() end
      end
  
      for i = 1, Party:GetMemberCount(), 1 do
        Party:GetMember(i).IsLinkDeadChanged = function()
          ReloadPlugin()
          --ChatOutput("Link dead changed")
        end
      end
    end
  end

  -- Refresh group info
  self.GroupEffectsAdded = function()

    -- Ignore all group effects
    if Settings.IDKMode then return end

    -- Go through all players
    for i = 1, Party:GetMemberCount(), 1 do

      -- Return if localplayer -> handled differently
      if self.Group[i].player:GetName() ~= self.Player:GetName() then
        self.callbacks[i] = {}
        self.callbacks[i]["EffectAdded"] = AddCallback(self.Group[i].effects, "EffectAdded", function(sender, args)
          local effect = self.Group[i].effects:Get(args.Index)
          if not(self.Group[i].currentEffects[effect:GetID()]) then
            self.Group[i].currentEffects[effect:GetID()] = true
            self.EffectUpdate(self, self.EffectAdded, {["effect"] = effect, ["groupID"] = i, ["name"] = self.Group[i].name})
          end
        end)
      end
    end
  end

  self.RefreshGroupData = function()
    if Party ~= nil then
      for i = 1, Party:GetMemberCount(), 1 do
        self.Group[i].effects = nil
        self.Group[i].currentEffects = nil
      end
      collectgarbage()
      for i = 1, Party:GetMemberCount(), 1 do
        self.Group[i].effects = self.Group[i].player:GetEffects()
        self.Group[i].currentEffects = {}
      end
    end
    self.GroupRats = {}
    if Party ~= nil and Party:GetAssistTargetCount() > 0 then
      for i = 1, Party:GetAssistTargetCount(), 1 do
        self.GroupRats[Party:GetAssistTarget(i):GetName()] = true
      end
    end
  end

  self.SetGroupInformation()
  self.SetGroupReloads()

  -- Party check
  if Party then 
    self.GroupEffectsAdded()
    ChatEventHandler:SetPartyData()
  end
end

function GroupEffectEvents:RemoveCallbacks()
  if Party then
    for i = 1, self.GroupSize, 1 do
      if self.Group[i].player:GetName() ~= self.Player:GetName() then 
        for key, value in pairs(self.callbacks[i]) do
          RemoveCallback(self.Group[i].effects, key, value)
        end
      end
    end
  end
end

function GroupEffectEvents:CheckGroupNameLUT(name)
  return(self.GroupNameLUT[name])
end

function GroupEffectEvents:CheckGroupRat(name)
  return(self.GroupRats[name])
end

function GroupEffectEvents:GetGroup()
  return self.Group
end