FightHandler = class(Turbine.UI.Control)
function FightHandler:Constructor(fightLines)

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()

    self.InstanceTier = Settings.InstanceTier
    self.InstanceID =  Settings.InstanceVersion

    self.InFight = false

    self.BossID = nil
    self.BossName = nil
    self.PhaseLines = nil
    self.EndLines = nil

    self.CurrentPhase = nil
    self.MaxPhases = nil

    self.StartTime = nil
    self.PhaseStartTime = nil

    self.FightLines = fightLines

    if not(self.chatCallback) then
        self.chatCallback = AddCallback(ChatEventHandler, "Say", function(line)
            self:TextUpdate(line)
        end)
    end

    if not(self.defeatCallback) then
        self.defeatCallback = AddCallback(ChatEventHandler, "Retreat", function(args)
            if self.InFight then
                self:HanldeEnd("", args.Name)
            end
        end)
    end
end

function FightHandler:TextUpdate(line)
    local name = GetFirstName(line)
    local cleanedLine = GetTextLine(line)

    --Find if the fight is starting
    if (not(self.InFight) or (self.InFight and not(self.player:IsInCombat()))) and self.FightLines[name] and self.FightLines[name][cleanedLine] then
        -- [1] Boss ID [2] Phases lines [3] End Line
        local StartTemp = self.FightLines[name][cleanedLine]
        self.BossID = StartTemp[1]
        self.PhaseLines = StartTemp[2]
        self.PhaseCount = tablelength(self.PhaseLines)
        self.EndLines = StartTemp[3]
        self.BossName = StartTemp[4]
        self.MaxPhases = tablelength(self.PhaseLines)
        self.StartTime = Turbine.Engine.GetGameTime()
        self.PhaseStartTime = Turbine.Engine.GetGameTime()
        self.InFight = true
        self.CurrentPhase = 1
        self.BossCount = StartTemp[5] or 1
        self.SuccessCount = 0
        if combatTimer then
            combatTimer:ChangeMode(true)
        end

        if InstanceController then
            InstanceController:FightStart({["Boss_Name"] = self.BossName, ["Boss_Tier"] = self.InstanceTier})
        end

        ChatOutput("<rgb=#f79000>Started encounter \"" .. self.BossName .. "\" on Tier " .. self.InstanceTier .." difficulty, good luck!")

        return
    end

    -- Return if the fight didn't start and not ongoing
    if not(self.InFight) then return end

    -- We are in a fight

    -- Check for phase ending
    if self.PhaseLines and self.PhaseLines[cleanedLine] then

        -- [1] Boss ID [2] Phase #
        local phaseTemp = self.PhaseLines[cleanedLine]

        if self.BossID == phaseTemp[1] and self.CurrentPhase < phaseTemp[2] then
            self.CurrentPhase = phaseTemp[2]
            local PhaseDuration = Turbine.Engine.GetGameTime() - self.PhaseStartTime
            ChatOutput("<rgb=#f79000>Completed phase " .. self.CurrentPhase - 1 .. " in " .. TimeDeltaToString(PhaseDuration))
            if Settings.EEnabled then
                effectsWindow:Message(8, "<rgb=#0008CFF>Phase ".. self.CurrentPhase .."</rgb>",nil,false)
            end
            self.PhaseStartTime = Turbine.Engine.GetGameTime()
            if InstanceController then
                InstanceController:FightPhase({["Boss_Name"] = self.BossName, ["Boss_Tier"] = self.InstanceTier, ["Phase"] = self.CurrentPhase})
            end
            return
        end
    end

    self:HanldeEnd(cleanedLine)
end

function FightHandler:HanldeEnd(cleanedLine, defeat)
    -- Check for fight ending
    local detectWipe = defeat ~= nil --and tablelength(self.EndLines) < 2

    if self.EndLines and self.EndLines[cleanedLine] or detectWipe then

        -- [1] Boss ID [2] Boss Name [3] Success
        local EndTemp
        if self.EndLines then
            EndTemp = self.EndLines[cleanedLine]
        end

        if detectWipe or self.BossID == EndTemp[1] then 

            local tempData = StatEventHandling:ReturnBossData(self.InstanceID, self.BossName, self.InstanceTier)
            local FightDuration = Turbine.Engine.GetGameTime() - self.StartTime
            
            -- We win
            if EndTemp and EndTemp[3] then
                self.SuccessCount = self.SuccessCount + 1
                if self.SuccessCount >= self.BossCount then
                    if tempData["FirstCompletion"] < 0 then
                        ChatOutput("<rgb=#f79000>Congratulations on your first successful clear on Tier " .. self.InstanceTier .. " difficulty! You have wiped ".. tempData["Wipes"] .. " times before this completion!")
                    end

                    if FightDuration < tempData["BestTime"] then
                        ChatOutput("<rgb=#f79000>Completed encounter " .. EndTemp[2] .. " in " .. TimeDeltaToString(FightDuration) .. ", new best time! You have completed this encounter ".. tempData["Completions"] + 1 .. " times on Tier " .. self.InstanceTier .. " difficulty!")
                    else
                        ChatOutput("<rgb=#f79000>Completed encounter " .. EndTemp[2] .. " in " .. TimeDeltaToString(FightDuration) .. ", your best time is: ".. TimeDeltaToString(tempData["BestTime"]) .."! You have completed this encounter ".. tempData["Completions"] + 1 .. " times on Tier " .. self.InstanceTier .. " difficulty!")
                    end
                    StatEventHandling:SaveBossData(self.InstanceID, EndTemp[2], self.InstanceTier, FightDuration, true, self.player)

                    if InstanceController then
                        local _, accData = StatEventHandling:GetData()
                        InstanceController:FightEnd({["Completion"] = false, ["Boss_Completion_Time"] = TimeDeltaToString(FightDuration), ["Boss_Tier"] = self.InstanceTier, ["Boss_Wipes"] = accData["BossData"][self.InstanceID][self.BossName][self.InstanceTier]["Wipes"]})
                    end
                else
                    return
                end
            end

            -- We Lost
            if detectWipe or not(EndTemp[3]) and FightDuration > 5 then
                ChatOutput("<rgb=#f79000>Wiped after " .. TimeDeltaToString(FightDuration) .. ", you have wiped " .. tempData["Wipes"] + 1 .. " times on Tier " .. self.InstanceTier .. " difficulty.")
                StatEventHandling:SaveBossData(self.InstanceID, self.BossName, self.InstanceTier, FightDuration, false, self.player)

                if InstanceController then
                    local _, accData = StatEventHandling:GetData()
                    InstanceController:FightEnd({["Boss_Completion"] = true, ["Boss_Completion_Time"] = TimeDeltaToString(FightDuration), ["Boss_Tier"] = self.InstanceTier, ["Boss_Wipes"] = accData["BossData"][self.InstanceID][self.BossName][self.InstanceTier]["Wipes"]})
                end
            end
            
            if combatTimer then
                combatTimer:ChangeMode(false)
            end
            if skillCounters then skillCounters:Clear() end
            self.InFight = false
        end
    end
end

function FightHandler:FightStatus()
    return self.InFight
end