ChatHandler = class(Turbine.UI.Control)
function ChatHandler:Constructor()
    Turbine.UI.Control.Constructor(self)
    local AlertsEnabled = Settings.AEnabled
    local SKTEnabled = Settings.SKTEnabled
    local SEnabled = Settings.SEnabled

    self.TyrantIcon = TyrantIcon
    self.KeenEyeIcon = KeenEyeIcon

    self.DamageLUT = DamageLUT
    self.SkillNames = E.SkillNames
    self.T = T

    self.SpecialNames = SpecialNames

    self.ChatFunctions = {}

    self.playerCombatTriggers = {}

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()

    self.callbacks = {}

    self.subscribers = {}

    self.ChatEvent = function(sender, event, args)
        if type(event)=="function" then
            event(args)
        else
            if type(event)=="table" then
                for i=1,#event do
                    if type(event[i])=="function" then
                        event[i](args)
                    end
                end
            end
        end
    end

    -- Set data
    self.leader = ""
    local player = self.player:GetName()

    self.UsedChatChannels = {
        [Turbine.ChatType.Raid] = true,
        [Turbine.ChatType.Fellowship] = true,
        [Turbine.ChatType.Say] = true,
        [Turbine.ChatType.Quest] = true,
        [Turbine.ChatType.Standard] = true,
        [Turbine.ChatType.Death] = true,
        [Turbine.ChatType.EnemyCombat] = true,
        [Turbine.ChatType.Advancement] = true,
        [Turbine.ChatType.PlayerCombat] = true,
    }

    -- Specific functions
    self.AlertMagic = function(message)

        -- Get sender name
        local sender = GetName(message)
        if sender then 

            -- Handler alerts
            if AlertsEnabled then
                local AlertTemp = self.A[sender]
                if AlertTemp ~= nil then 
                    for key, value in pairs(AlertTemp) do
                        if string.find(string.gsub(message, "[%p%c%s]", ""), string.gsub(key, "[%p%c%s]", "")) then
                            messageWindow:Message(value[1], value[2], value[3], value[4])
                            if value[5] ~= nil then Sounds:PlaySoundEffect(value[5]) end
                            if value[6] and targetBox then
                                local icon = nil
                                if value[7] ~= nil and value[7] ~= true then
                                icon = value[7]
                                end
                                targetBox:ShowBox(GetFirstName(message), icon)
                            end
                        end
                    end
                end
            end

            -- Handle timers
            if SEnabled then 
                local TimerTemp = self.T[sender]
                if TimerTemp ~= nil then
                    for key, value in pairs(TimerTemp) do
                        if string.find(message, key) then
                            skillCounters:NewTimer(value[1], value[2], nil, nil, nil, value[3])
                            return(false)
                        end
                    end
                end
            end
        end
    end

    self.CheckCommandPermission = function(message, senderName)
        if senderName == self.leader or 
        ((player == self.leader or self.SpecialNames[player]) and 
        (string.find(message, "%[To Fellowship%]") or 
        string.find(message, "%[To Group%]") or 
        string.find(message, "%[To Raid%]"))) or 
        self.SpecialNames[senderName] or 
        GroupEffectsHandler:CheckGroupRat(senderName) or
        ((self.SpecialNames[player] or GroupEffectsHandler:CheckGroupRat(player)) and (string.find(message, "%[To Fellowship%]" or string.find(message, "%[To Group%]") or 
        string.find(message, "%[To Raid%]")))) then
            return(true)
        else
            return(false)
        end
    end

    self.cleanPartyMessage = function(message)

        -- Fellowship
        if string.find(message, "%[To Fellowship%]") then
            return string.match(message, "%[To Fellowship%] (.*)")
        elseif string.find(message, "%[Fellowship%]") then
            local name, message = string.match(message, "%[Fellowship%] (.*): (.*)")
            return message
        -- Group
        elseif string.find(message, "%[To Group%]") then
            return string.match(message, "%[To Group%] (.*)")
        elseif string.find(message, "%[Group%]") then
            local name, message = string.match(message, "%[Group%] (.*): (.*)")
            return message
        -- Raid
        elseif string.find(message, "%[To Raid%]") then
            return string.match(message, "%[To Raid%] (.*)")
        elseif string.find(message, "%[Raid%]") then
            local name, message = string.match(message, "%[Raid%] (.*): (.*)")
            return message
        end

        return message
    end

    -- Chat handling functions
    self.PartyChatHandling = function(message)
        -- Handle impossible messages :)
        if Party == nil then return end

        -- Check name of chat sender
        local senderName = GetName2(message)

        self.ChatEvent(self, self.Party, self.cleanPartyMessage(message):gsub("\n", ""))

        -- Handle raid commands
        if self.CheckCommandPermission(message, senderName) then   

            -- Countdown command        
            if string.find(message, "countdown") then
                local number = tonumber(string.sub(message, string.find(message, "countdown") + 10, string.len(message)))
                if number then                 
                    countdownWindow:Countdown(number)
                end
            end

            -- Timer command
            if string.find(message, "start timer") then
                local timerName = string.sub(message, string.find(message, "start timer") + 12, string.find(message, ",") - 1)
                local number = tonumber(string.sub(message, string.find(message, ",") + 2, string.len(message)))
                if number and SEnabled then skillCounters:NewTimer(number, timerName, true) end
            end
                
            -- Note commands
            if string.find(message, "note") then
                -- Add command
                if(string.find(message, "noteAdd")) then
                    NoteHandler:AddText(string.sub(message, string.find(message, "noteAdd") + 8, string.len(message)))
                end

                -- Clear command
                if(string.match(message, "noteClear")) then
                    NoteHandler:ClearText()
                end
            end

            -- Ready check events start
            if(string.find(message, " Initiate Readycheck")) then
                ReadyHandler:MakeNewReadyCheck()
            end

            -- Reload
            if(string.find(message, " Force Reload")) then
                ReloadPlugin()
            end
        end

        if SuperSpecialNames[senderName] or (SuperSpecialNames[self.player:GetName()] and (string.find(message, "%[To Fellowship%]") or string.find(message, "%[To Raid%]"))) then
            local clearedMessage = self.cleanPartyMessage(message):gsub("\n", "")

            if clearedMessage == "Curse of the Selector" then
                MouseCurse = UI.MouseDecoy()
            end

            if clearedMessage == "Flashbang" then
                FlashBang = UI.FlashBang()
            end

            if clearedMessage == "Wink" then
                Wink = UI.Wink()
            end

            if clearedMessage == "Helloooo" then
                DriveBy = UI.DriveBy()
            end

            if clearedMessage == "I like trains" then
                TrainBy = UI.TrainBy()
            end

            if clearedMessage == "Peekaboo" then
                Peekaboo = UI.Peekaboo()
            end

            if clearedMessage == "Make it rain" then
                MakeItRain = UI.MakeItRain()
            end
        end

        -- Amazing
        if senderName == "Arandour" or player == "Arandour" then
            if string.find(message, "Amazing") then
                Sounds:PlaySoundEffect(SoundEffects["Amazing"])
            end
        end


        -- Handle self ready
        if ReadyHandler:ReadyCheckActive()  then
            if string.find(message, " Ready") and not(string.find(message, " Initiate Readycheck" )) then
                if GetName2(message) == "Ready" then 
                    --ReadyHandler:ActionCheck()
                end
            end
        end

        -- Handle readycheck udpates
        if ReadyHandler:ReadyCheckActive() then
            if string.find(message, " Ready") and not(string.find(message, " Initiate Readycheck" )) then
                local playerName = ""
                for word in string.gmatch(message, "%S+") do
                    if(word == "Not" or word == "Ready") then
                        break
                    end  
                    playerName = word
                end
                playerName = GetName2(message)
                if playerName == "Ready" then
                    playerName = self.player:GetName()
                    ReadyHandler:ShowOverview()
                end

                if string.find(message, " Not Ready") then
                    ReadyHandler:ReadyUpdate(playerName, false)
                else
                    ReadyHandler:ReadyUpdate(playerName, true)
                end
            end
        end
    end

    self.ChatFunctions[Turbine.ChatType.Fellowship] = self.PartyChatHandling
    self.ChatFunctions[Turbine.ChatType.Raid] = self.PartyChatHandling

    self.SayChatHandling = function(message)
        -- Old clear command
        if string.find(message, "You say, 'clear'") then ClearUI() end

        self.ChatEvent(self, self.Say, message)

        self.ChatEvent(self, self.SayLine, GetTextLine(message))
    end

    self.ChatFunctions[Turbine.ChatType.Say] = self.SayChatHandling

    self.QuestChatHandling = function(message)
        -- Instance controller handling
        if InstanceController then InstanceController:CheckInstanceState(message) end

        -- Loot window handling
        if string.sub(message, 1, string.find(message, "%s") - 1) == "Completed:" then
            LootHandler:QuestParsing(string.sub(message, string.find(message, "%s") + 1, string.len(message) - 1))
        end
    end

    self.ChatFunctions[Turbine.ChatType.Quest] = self.QuestChatHandling

    -- Lootlock

    self.StandardChatHandling = function(message)
        self.ChatEvent(self, self.Standard, message)

        if string.find(message, PluginString) then return end

        -- Instancecontroller chat handling
        if InstanceController then InstanceController:CheckInstanceState(message) end

        -- Fight chat chandling
        if Fight then Fight:TextUpdate(message) end

        -- Loot window chat handling
        LootHandler:LootlockParsing(message)
        --HandleNewLock(message)

        -- Handle party change reloading
        if string.find(message, "Fellowship") or string.find(message, "Raid") or string.find(message, "dismiss") or string.find(message, "raid") or string.find(message, "fellowship") then
            if string.find( message, " has joined your Fellowship." ) or 
            string.find( message, " has joined your Raid." ) or
            string.find( message, "You have joined a Fellowship." ) or
            string.find( message, "You have joined a Raid." ) or
            string.find( message, " has left your Fellowship." ) or
            string.find( message, " has left your Raid." ) or
            string.find( message, "You leave your Fellowship." ) or
            string.find( message, "You leave your Raid." ) or
            string.find( message, "You dismiss " ) or
            string.find( message, " from the Raid." ) or
            string.find( message, " has been dismissed from your Fellowship." ) or
            string.find( message, " has been dismissed from your Raid." ) or
            string.find( message, "You have been dismissed from your Raid." ) or
            string.find( message, "You have been dismissed from your Fellowship." ) or
            string.find( message, "Your Fellowship has been disbanded." ) or
            string.find( message, "Your Raid has been disbanded." ) or
            string.find( message, "You have joined the chat room for your raid, raid chat is now available." ) or
            string.find( message, "You have joined the chat room for your Fellowship, Fellowship chat is now available." ) or
            string.find( message, "the leader of the raid") or
            string.find( message, "the leader of the fellowship") then
                ReloadPlugin()
            end
        end

        -- Nice
        if string.find(message, "rolled (100): 69") and AlertsEnabled then
            messageWindow:Message(3, "Nice!", nil, false)
        end

        -- Handle track alerts
        if string.find(message, "You feel as though you are being followed...") and AlertsEnabled then
            messageWindow:Message(3, "You are being tracked!", nil, false)
            StatEventHandling:AddTrack()
        end
    end

    self.ChatFunctions[Turbine.ChatType.Standard] = self.StandardChatHandling

    --  You have freed Superior Uruk Guard from a daze!
    -- Charged Fulgurite Rune-stone has freed Superior Uruk Guard from a daze!

    self.DeathChatHandling = function(message)

        if string.find(message, "freed") then
            local culprit, target, ccType = string.match(message, "(.*) ha[sv]e? freed (.*) from a (.*)!")
            self.ChatEvent(self, self.CCBreak, {["Culprit"] = string.gsub(culprit, "You", self.player:GetName()), ["Target"] = target, ["Type"] = ccType})
        end

        -- Player death
        if string.find(message, "You succumb to your wounds.") then 
            if Fight then Fight:TextUpdate(message) end
            self.ChatEvent(self, self.Retreat, {["Name"] = self.player:GetName()})
        elseif string.find(message, "Your fellow,") then
            self.ChatEvent(self, self.Retreat, {["Name"] = string.match(message, "Your fellow, (.*) has been defeated!")})
        end

        InstanceController:ParsePlus(message)

        -- KB handling
        if string.find(message, "Your mighty blow defeated") then
            local kbName = string.gsub(string.match(message, "Your mighty blow defeated (.+)%."),"^[Tt]he ","")
            self.ChatEvent(self, self.Defeated, {["Name"] = kbName})
            StatEventHandling:AddKB(kbName)
        elseif string.find(message, "defeated") then
            local source, name = string.match(message, "(.*) defeated (.+)%.")
            if name then name = string.gsub(name, "^[Tt]he ","") end
            self.ChatEvent(self, self.Defeated, {["Source"] = source, ["Name"] =  name or string.match(message,"(.*) has been defeated.")})
        elseif string.find(message, "Your mighty blow topples") then
            StatEventHandling:AddKB(string.match(message, "Your mighty blow topples (.+)%."), true)
        end

        -- Getting killed handling
        if string.find(message, " incapacitated you.") then
            local source = string.match(message, "(.+) incapacitated you.")
            StatEventHandling:AddDeath(source, false)
            self.ChatEvent(self, self.Defeated, {["Death_Source"] = source, ["Name"] =  self.player:GetName()})
        end

        -- Misadventure death handling
        if string.find(message, "You have been incapacitated by misadventure.") then
            StatEventHandling:AddDeath(nil, true)
            self.ChatEvent(self, self.Defeated, {["Death_Source"] = "Misadventure", ["Name"] =  self.player:GetName()})
        end
    end

    self.ChatFunctions[Turbine.ChatType.Death] = self.DeathChatHandling

    self.EnemyCombatChatHandling = function(message)
        --local source, _, skillName, rest = string.match(string.gsub(string.gsub(message,"<rgb=#......>(.*)</rgb>","%1"),"^%s*(.-)%s*$", "%1"),"^(.*) scored a (.*)hit with (.*) on (.*) for (.-) (.*)%.$")
        local source, _, skillName, rest = string.match(string.gsub(string.gsub(message,"<rgb=#......>(.*)</rgb>","%1"),"^%s*(.-)%s*$", "%1"),"^(.*) scored a (.*)hit with (.*) on (.*)%.$")
        if skillName then
            if rest then
                local target, _ = string.match(rest,"^(.*) for (.*)$")
                if target then
                    self.ChatEvent(self, self.EnemyCombat, {["SkillName"] = skillName, ["Target"] = string.gsub(target, "^[Tt]he ", "")})
                else
                    self.ChatEvent(self, self.EnemyCombat, {["SkillName"] = skillName, ["Target"] = string.gsub(rest, "^[Tt]he ", "")})
                end
                
            else
                self.ChatEvent(self, self.EnemyCombat, {["SkillName"] = skillName, ["Target"] = ""})
            end
        end   
        if self.DamageLUT[skillName] then
            if skillName == "Steadfast Barrage" then
                if AlertsEnabled then
                    messageWindow:Message(2, "Barrage from: <rgb=#FF0000>".. source .. "</rgb>", self.TyrantIcon, false)
                end

                if SKTEnabled then
                    PVPCDTracker:NewTimer(30, source, nil, true)
                end
            end
        end
    end

    self.ChatFunctions[Turbine.ChatType.EnemyCombat] = self.EnemyCombatChatHandling

    self.PlayerCombatChatHandler = function(message)
        local cleanedMessage = string.gsub(string.gsub(message,"<rgb=#......>(.*)</rgb>","%1"),"^%s*(.-)%s*$", "%1")
        --local source,_,skillName,target,_ = string.match(cleanedMessage,"^(.*) scored a (.*)hit with (.*) on (.*) for (.-) (.*)%.$")
        local source, _, skillName, rest = string.match(cleanedMessage,"^(.*) scored a (.*)hit with (.*) on (.*)%.$")

        if self.tools then
            self.tools:ParseUpdate(cleanedMessage)
        end

        for key, value in pairs(self.subscribers) do key:ParseUpdate(cleanedMessage) end

        if skillName then
            if rest then
                local target, _ = string.match(rest,"^(.*) for (.*)$")
                if target then
                    self.ChatEvent(self, self.PlayerCombat, {["SkillName"] = skillName, ["Target"] = target})
                else
                    self.ChatEvent(self, self.PlayerCombat, {["SkillName"] = skillName, ["Target"] = string.gsub(rest, "^[Tt]he ", "")})
                end
                
            else
                self.ChatEvent(self, self.PlayerCombat, {["SkillName"] = skillName, ["Target"] = ""})
            end
            return
        end       
        --Fighter of Shadow applied a heal to Rogrim restoring 34,476 points to Morale.
        --Mending Verse applied a heal to Rotorin restoring 2,946 points to Morale.
        --Rotorin applied a benefit with Mending Verse on Rotorin.
        local skillName, _, target, _ = string.match(cleanedMessage,"^(.*) applied a (.*)heal to (.*) restoring (.*) points to Morale%.$")
        if skillName then
            self.ChatEvent(self, self.PlayerHeal, {["SkillName"] = skillName, ["Target"] = target})
            return
        end 
        
        -- source, type, skillName, target, number
        local _, _, skillName, target, _ = string.match(cleanedMessage, "^(.*) applied a (.*)heal with (.*) to (.*) restoring (.*) points to Morale%.$")
        if skillName then
            self.ChatEvent(self, self.PlayerHeal, {["SkillName"] = skillName, ["Target"] = target})
            return
        end

        local source, _, skillName, target = string.match(cleanedMessage,"^(.*) applied a (.*)benefit with (.*) on (.*)%.$")
        if skillName then
            self.ChatEvent(self, self.PlayerBenefit, {["SkillName"] = skillName, ["Target"] = target, ["Source"] = source})
            return
        end 
    end

    self.ChatFunctions[Turbine.ChatType.PlayerCombat] = self.PlayerCombatChatHandler

    self.AdvancementHandling = function(message)
        if string.find(message, "You have acquired") then
            Roles:ParseTraits(message)
        end
    end

    self.ChatFunctions[Turbine.ChatType.Advancement] = self.AdvancementHandling

    -- Set parse function
    self.callbacks["Received"] = AddCallback( Turbine.Chat, "Received", function(sender, args)
        if self.ChatFunctions[args.ChatType] then
            self.ChatFunctions[args.ChatType](args.Message)
        end
    end)
end

function ChatHandler:RemoveCallbacks()
    for key, value in pairs(self.callbacks) do
        RemoveCallback(Turbine.Chat, key, value)
    end
end

function ChatHandler:SetPartyData()
    self.leader = Party:GetLeader():GetName()
end

function ChatHandler:SubscribeTools(tools)
    self.tools = tools
end

function ChatHandler:TogglePlayerCombat(value)
    if value == nil and #self.subscribers <= 0 and self.tools == nil then
        --self.UsedChatChannels[Turbine.ChatType.PlayerCombat] = nil
    else
        --self.UsedChatChannels[Turbine.ChatType.PlayerCombat] = true
    end
end

function ChatHandler:Subscribe(object)
    self.subscribers[object] = true
end

function ChatHandler:UnSubscribe(object)
    self.subscribers[object] = false
end

--[[
Turbine.Chat.Received = function(sender,args)
    if not(UsedChatChannels[args.ChatType]) then return end
    if args.ChatType == Turbine.ChatType.Fellowship or args.ChatType == Turbine.ChatType.Raid then
        if Party == nil then return end
        local leader = Party:GetLeader():GetName()
        local player = Player:GetName()
        local senderName = GetName2(args.Message) 
        if CheckCommandPermission(args.Message, senderName, player, leader) then           
            if(string.find(args.Message, "countdown")) then
                local number = tonumber(string.sub(args.Message, string.find(args.Message, "countdown") + 10, string.len(args.Message)))
                if number ~= nil then                 
                    countdownWindow:Countdown(number)
                end
            end
                    
            if(string.find(args.Message, "noteAdd")) then
                noteWindow.noteWindow:AppendText(string.sub(args.Message, string.find(args.Message, "noteAdd") + 8, string.len(args.Message)))
            end

            if(string.match(args.Message, "noteClear")) then
                noteWindow.noteWindow:SetText("")
                noteWindow:SetVisible(true)
            end

            --Ready check events start
            if(string.find(args.Message, " Initiate Readycheck")) then
                RcWindow:MakeReadyCheck()
                ReadyCheck:SetVisible(false)
                RCButtonWindow:SetVisible(false)
                ReadyCheck:MakeNewReadyCheck()
                Sounds:PlaySoundEffect(SoundEffects["Rc"]) 
            end

            if(string.find(args.Message, " Force Reload")) then
                ReloadPlugin()
            end
        end

        if senderName == "Arandour" or player == "Arandour" then
            if string.find(args.Message, "Amazing") then
                Sounds:PlaySoundEffect(SoundEffects["Amazing"])
            end
        end


        -- Handle self ready
        if RcWindow:OngoingReadycheck()  then
            if string.find(args.Message, " Ready") and not(string.find(args.Message, " Initiate Readycheck" )) then
                if GetName2(args.Message) == "Ready" then 
                    RcWindow:ActionCheck()
                    ReadyCheck:SetVisible()
                end
            end
        end

        -- Handle readycheck udpates
        if ReadyCheck:ReadyCheckActive() then
            if string.find(args.Message, " Ready") and not(string.find(args.Message, " Initiate Readycheck" )) then
                local playerName = ""
                for word in string.gmatch(args.Message, "%S+") do
                    if(word == "Not" or word == "Ready") then
                        break
                    end  
                    playerName = word
                end
                playerName = GetName2(args.Message)
                if playerName == "Ready" then
                    playerName = Player:GetName()
                end

                if string.find(args.Message, " Not Ready") then
                    ReadyCheck:UpdateReadycheckMember(playerName, false)
                else
                    ReadyCheck:UpdateReadycheckMember(playerName, true)
                end
            end
        end

        
    elseif args.ChatType == Turbine.ChatType.Say then
        --ChatOutput("Say"..args.Message)
        --ReloadPlugin()
        if string.find(args.Message, "You say, 'clear'") then
            ClearUI()
        end
        if(string.find(args.Message, "countdown")) then
            if Party ~= nil then
                local senderName = GetName2(args.Message) 
                local leader = Party:GetLeader():GetName()
                local player = Player:GetName()
                if senderName == leader or (player == leader and (string.find(args.Message, "You say"))) or (SpecialNames[player] and (string.find(args.Message, "You say"))) then           
                    local number = tonumber(string.match(string.sub(args.Message, string.find(args.Message, "countdown") + 10, string.len(args.Message)), "%d+"))
                    if number ~= nil then                 
                        countdownWindow:Countdown(number)
                    end
                end
            end
        end
        --TestMessage(args.Message)
        AlertMagic(args.Message)
        if fight then fight:TextUpdate(args.Message) end
        --HandleNewLock(args.Message)
    elseif args.ChatType == Turbine.ChatType.Quest then
        if InstanceController ~= nil then InstanceController:CheckInstanceState(args.Message) end
        --ChatOutput(string.sub(args.Message, 1, string.find(args.Message, "%s")) .. "a")
        --ChatOutput(string.sub(args.Message, string.find(args.Message, "%s") + 1, string.len(args.Message)))
        --ChatOutput(args.Message)

        if string.sub(args.Message, 1, string.find(args.Message, "%s") - 1) == "Completed:" then
            HandleQuestLocks(string.sub(args.Message, string.find(args.Message, "%s") + 1, string.len(args.Message) - 1))
        end
    elseif args.ChatType == Turbine.ChatType.Standard then
        --ChatOutput(args.Message)
        --ChatOutput(GetChestName(args.Message))
        if InstanceController ~= nil then InstanceController:CheckInstanceState(args.Message) end
        if fight then fight:TextUpdate(args.Message) end
        HandleNewLock(args.Message)

        if string.find(args.Message, "Raid") or string.find(args.Message, "Fellowship") then
            ReloadPlugin()
        end
        
        if string.find( args.Message, " has joined your Fellowship." ) or 
        string.find( args.Message, " has joined your Raid." ) or
        string.find( args.Message, "You have joined a Fellowship." ) or
        string.find( args.Message, "You have joined a Raid." ) or
        string.find( args.Message, " has left your Fellowship." ) or
        string.find( args.Message, " has left your Raid." ) or
        string.find( args.Message, "You leave your Fellowship." ) or
        string.find( args.Message, "You leave your Raid." ) or
        string.find( args.Message, "You dismiss " ) or
        string.find( args.Message, " from the Raid." ) or
        string.find( args.Message, " has been dismissed from your Fellowship." ) or
        string.find( args.Message, " has been dismissed from your Raid." ) or
        string.find( args.Message, "You have been dismissed from your Raid." ) or
        string.find( args.Message, "You have been dismissed from your Fellowship." ) or
        string.find( args.Message, "Your Fellowship has been disbanded." ) or
        string.find( args.Message, "Your Raid has been disbanded." ) or
        string.find( args.Message, "You have joined the chat room for your raid, raid chat is now available." ) or
        string.find( args.Message, "You have joined the chat room for your Fellowship, Fellowship chat is now available." ) or
        string.find( args.Message, "the leader of the raid") or
        string.find( args.Message, "the leader of the fellowship") then
            ReloadPlugin()
        end

        if string.find(args.Message, "You feel as though you are being followed...") and AlertsEnabled then
            messageWindow:Message(3, "You are being tracked!", nil, false)
            AddTrack()
        end
    elseif args.ChatType == Turbine.ChatType.Death then
        if string.find(args.Message, "You succumb to your wounds.") then 
            if fight then fight:TextUpdate(args.Message) end
        end
        local name = nil
        if string.find(args.Message, "defeated") then
            name = string.match(args.Message, "defeated ".."(.+)".. "%.")
        end
        if string.find(args.Message, "has been defeated") then
            name = string.match(args.Message,"(.+)" .. "% has been defeated.")
        end
        if AlertsEnabled and name ~= "" then 
            rezTracker:Death(name)
            if targetBox and GroupEffectsHandler:CheckGroupNameLUT(name) then targetBox:ShowBox(name, DeathSkullIcon) end
         end

        if string.find(args.Message, "Your mighty blow defeated the") then
            if string.find(args.Message, "Your mighty blow defeated") then
                local kb = string.match(args.Message, "Your mighty blow defeated the (.+)%.")
                AddKB(kb)
            elseif string.find(args.Message, "Your mighty blow defeated The") then
                local kb = string.match(args.Message, "Your mighty blow defeated The (.+)%.")
                AddKB(kb)

            end
        elseif string.find(args.Message, "Your mighty blow topples") then
            local kb = string.match(args.Message, "Your mighty blow topples (.+)%.")
            AddKB(kb, true)
        end
        if string.find(args.Message, " incapacitated you.") then
            local kb = string.match(args.Message, "(.+) incapacitated you.")
            AddDeath(kb, false)
        end

        if string.find(args.Message, "You have been incapacitated by misadventure.") then
            AddDeath(nil, true)
        end
        --ChatOutput(args.Message)
    elseif args.ChatType == Turbine.ChatType.EnemyCombat then
        local source,_,skillName,target,_ = string.match(string.gsub(string.gsub(args.Message,"<rgb=#......>(.*)</rgb>","%1"),"^%s*(.-)%s*$", "%1"),"^(.*) scored a (.*)hit with (.*) on (.*) for (.-) (.*)%.$")

        if DamageLUT[skillName] then
            if skillName == "Steadfast Barrage" then
                if Settings.AEnabled then
                    messageWindow:Message(2, "Barrage from: <rgb=#FF0000>".. source .. "</rgb>", TyrantIcon, false)
                end

                if targetBox then
                    targetBox:ShowBox(source, KeenEyeIcon)
                end

                if Settings.SKTEnabled then
                    PVPCDTracker:NewTimer(30, source, nil, true)
                end
            end

            local alert = E.SkillNames[skillName]
            if alert ~= nil then
                messageWindow:Message(alert[1], alert[2], alert[3], alert[4])
            end

            local timer = TDamage[skillName]
            if timer ~= nil then
                skillCounters:NewTimer(timer[1], timer[2])
            end
        end
    end
end
[01/09 01:49:32 AM] Shaktur's Silver Chest resets in: 0d - 7h - 10m.
Gâdh-and-Shum's Chest - Tier 1: You have 4 favoured completions remaining.
--]]

function CompareIt(a, b)
    --ChatOutput(a)
    --ChatOutput(b)
    if string.find(a, b) then return true end
end

function AlertMagic(message)
    --ChatOutput(message) 
    --ChatOutput(sender)  
    local sender = GetName(message)
    --ChatOutput(sender)
    if sender ~= nil then 
        --ChatOutput(sender)
        --if Settings.LEnabled then
        --    logWindow:Message(sender, false)
        --end
        if Settings.AEnabled then
            local temp1 = A[sender]
            if temp1 ~= nil then 
                for key, value in pairs(temp1) do
                    if string.find(string.gsub(message, "[%p%c%s]", ""), string.gsub(key, "[%p%c%s]", "")) then
                        messageWindow:Message(value[1], value[2], value[3], value[4])
                        if value[5] ~= nil then Sounds:PlaySoundEffect(value[5]) end
                        if value[6] and targetBox then
                            local icon = nil
                            if value[7] ~= nil and value[7] ~= true then
                              icon = value[7]
                            end
                            targetBox:ShowBox(GetFirstName(message), icon)
                        end
                    end
                end
            end
        end

        if Settings.SEnabled then 
            local temp2 = T[sender]
            if temp2 ~= nil then
                for key, value in pairs(temp2) do
                    if string.find(message, key) then
                        skillCounters:NewTimer(value[1], value[2])
                        return(false)
                    end
                end
            end
        end
    end
end

function GetName(message)
    local sender = nil
    local x = message:sub(1, message:find("%s"))
    for word in string.gmatch(x:sub(8), "%a+") do
        --ChatOutput(word)
        if(word == "Select") then
            break
        end  
        --ChatOutput(sender)
        sender = word
    end
    return(sender)
end

function GetNameDemo(message)
    local sender = nil
    for word in string.gmatch(message, "%a+") do
        --ChatOutput(word)
        if(word == "Select") then
            --break
        end  
        --ChatOutput(sender)
        sender = word
    end
    return(sender)
end

function GetFirstName(message)
    if "You" == string.sub(message, 1, 3) then return "You" end
    local isolateIID = nil
    if string.find(message, "says") then
        isolateIID = string.sub(message, 1, string.find(message, "says") - 2)
    elseif string.find(message, "shouts") then
        isolateIID = string.sub(message, 1, string.find(message, "shouts") - 2)
    else
        return ""
    end
    if isolateIID and string.find(isolateIID, ">") then 
        return(string.sub(isolateIID, string.find(isolateIID, ">") + 1, string.find(isolateIID, "<\\Select>") - 1))
    else
        return ""
    end
end

function TestMessage(message)
    ChatOutput(GetFirstName(message))
    ChatOutput(GetTextLine(message))
    local isolateIID = nil
    if string.find(message, "says") then
        isolateIID = string.sub(message, 1, string.find(message, "says") - 2)
    elseif string.find(message, "shouts") then
        isolateIID = string.sub(message, 1, string.find(message, "shouts") - 2)
    end
    ChatOutput("<Select:IID:" .. string.sub(isolateIID, string.find(isolateIID, "IID:") + 4, string.find(isolateIID, ">") - 1) .. ">YOUR MOM<\\Select>")

end

function GetTextLine(message)
    if string.find(message, "say") then
        if string.find(message, "You say") then
            return string.sub(message,string.find(message, "say") + 6, string.len(message) - 2)
        else
            return string.sub(message,string.find(message, "say") + 8, string.len(message) - 3)
        end
    elseif string.find(message, "shouts") then
        return string.sub(message, string.find(message, "shouts") + 10, string.len(message) - 3)
    else return message end
end

function GetName2(message)
    local sender = nil
    local breakBool = false
    for word in string.gmatch(message, "%a+") do
        if(word == "Select")then
            if not(breakBool) then
                breakBool = true
            else
                break
            end
        end  
        sender = word
    end
    return(sender)
end

function GetChestName(message, raid)
    local sender = ""
    for key, value in pairs(SpecialChestNames) do
        if (string.find( message,key)) then
            return(value)
        end
    end
    for word in string.gmatch(message, "%S+") do
        if(word == "Mithril") then
            --ChatOutput(word)
            break
        end  
        sender = sender..word
    end
    if sender == "" then
        for key, value in pairs(SpecialChestNames) do
            if (string.find( message,key)) then
                return(value)
            end
        end
    end
    --return(sender)
end

function GetRaidChestName(message)
    local sender = nil
    for key, value in pairs(SpecialRaidChestNames) do
        if (string.find( message,key)) then
            return(value)
        end
    end
    --[[
    for word in string.gmatch(message, "%S+") do
        if(word == "-") then
            --ChatOutput(word)
            break
        end  
        sender = sender..word
    end
    return(sender)]]
end

function HandleNewLock(message)
    
    -- Check if we have a T2/T3 raid opening
    if string.find(message, "Tier") then

        -- Check for an instance lock
        if NormaleChest(message) then return end

        -- Check for raid chest
        local chestname = GetRaidChestName(message)

        -- Check for a hit
        if not(chestname) then return end

        -- Check if we have a T1 raid opening
        if string.find(message, "Tier 1") then


            HandleNullLootTable()
            chestname = "T1 " .. chestname
            if LootLocks.T1RaidLocks == nil then
                LootLocks.T1RaidLocks = {}
            end
            if LootLocks.T1RaidLocks[chestname] == nil then
                --ChatOutput("Boss added")
                LootLocks.T1RaidLocks[chestname] = {}
            end
            if LootLocks.T1RaidLocks[chestname][Player:GetName()] == nil then
                LootLocks.T1RaidLocks[chestname][Player:GetName()] = 0
            end
        else

            -- Higher than T1
            local number = string.sub(message, string.find(message, "Tier") + 5, string.find(message, "Tier") + 5)

            -- No instance chest but raid chest
            if number == nil then number = "?" end
            chestname = "T".. number .. " " ..chestname

            HandleNullLootTable()

            -- Handle table values
            if LootLocks.RaidLocks == nil then
                LootLocks.RaidLocks = {}
            end
            if LootLocks.RaidLocks[chestname] == nil then
                LootLocks.RaidLocks[chestname] = {}
            end
            if LootLocks.RaidLocks[chestname][Player:GetName()] == nil then
                LootLocks.RaidLocks[chestname][Player:GetName()] = 0
            end
        end

    -- check if we have a favoured T3 opening with a weird fucking name
    elseif string.find(message, "Mithril Chest") or string.find(message, "Mithril Reward") then
        NormaleChest(message)
    end 
end

function NormaleChest(message)
    -- Check if we already have this char in the loot table
    local chestName = GetChestName(message)

    if chestName == nil then
        return(nil)
    end

    HandleNullLootTable()

    local number = nil
    if string.find(message, "Mithril Chest") or string.find(message, "Mithril Reward") then 
        number = "3"
    else
        number = string.sub(message, string.find(message, "Tier") + 5, string.find(message, "Tier") + 5)
    end

    if number == nil then number = "?" end
    chestName = "T".. number .. " " .. chestName

    if number == "1" then
        if LootLocks.T1Locks == nil then
            LootLocks.T1Locks = {}
        end
        if LootLocks.T1Locks[chestName] == nil then
            --ChatOutput("Boss added")
            LootLocks.T1Locks[chestName] = {}
        end
        if LootLocks.T1Locks[chestName][Player:GetName()] == nil then
            LootLocks.T1Locks[chestName][Player:GetName()] = 0
        end

    else
        --ChatOutput(message)
        --ChatOutput(chestName)
        if LootLocks.Locks == nil then
            LootLocks.Locks = {}
        end
        if LootLocks.Locks[chestName] == nil then
            --ChatOutput("Boss added")
            LootLocks.Locks[chestName] = {}
        end
        if LootLocks.Locks[chestName][Player:GetName()] == nil then
            LootLocks.Locks[chestName][Player:GetName()] = 1
        else
            LootLocks.Locks[chestName][Player:GetName()] = 0
        end

        return true
    end
end

function HandleNullLootTable()
        
    if LootLocks.Characters == nil then
        LootLocks.Characters = {}
    end

    if LootLocks.Characters[Player:GetName()] == nil then
        LootLocks.Characters[Player:GetName()] = true
    end
end

function HandleQuestLocks(message)

    local questName = QuestNamesToLootNames[message]

    local lockType = nil

    if questName == nil then return end 

    HandleNullLootTable()

    -- Daily
    if questName[2] == 1 then
        lockType = "DailyLocks"

    --Weekly
    elseif questName[2] == 2 then
        lockType = "WeeklyLocks"
    end

    if lockType == nil then return end

    if LootLocks[lockType][questName[1]] == nil then
        LootLocks[lockType][questName[1]] = {}
    end 

    if LootLocks[lockType][questName[1]][Player:GetName()] == nil then
        LootLocks[lockType][questName[1]][Player:GetName()] = 0
    end 
end

function CheckCommandPermission(message, senderName, player, leader)
    if senderName == leader or 
    (player == leader and (string.find(message, "%[To Fellowship%]") or 
    string.find(message, "%[To Raid%]"))) or 
    SpecialNames[senderName] or 
    GroupEffectsHandler:CheckGroupRat(senderName) or
    ((SpecialNames[player] or GroupEffectsHandler:CheckGroupRat(player)) and (string.find(message, "%[To Fellowship%]") or 
    string.find(message, "%[To Raid%]"))) then
        return(true)
    else
        return(false)
    end
end

        --[[
        if chestname == nil then
            local chestName2 = GetChestName(message)
            if chestName2 ~= nil and  string.find(message, "Tier 3") then
                HandleNullLootTable()
                NormaleChest(message)
                return(nil)
            else
                return(nil)
            end
        end
        --]]