ReadyCheckHandler = class(Turbine.UI.Control)
function ReadyCheckHandler:Constructor()
    Turbine.UI.Control.Constructor(self)

    if not(Turbine.Gameplay.LocalPlayer.GetInstance():GetParty()) then return end

    self.readyActive = false
    self.readyStartTime = 0

    self.readyOverview = ReadyWindow(self)
    self.readyAcceptanceWindow = ReadyCheckWindow()
    self.ChatReadyButton = ReadyButtonWindow()

    self.ExecuteUpdate = function(delta)
        self.readyStartTime = self.readyStartTime + delta
    
        self.readyOverview:SetTimerBar((self.maxReadyTime - self.readyStartTime) / 60)
    
        if self.maxReadyTime < self.readyStartTime then
            self.readyOverview:OutputState()
            if self.updateCallback then if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end end
            self.updateCallback = nil
        end
    end
end

function ReadyCheckHandler:MakeNewReadyCheck()
    -- Make sure the button to start is invisble 
    self.ChatReadyButton:SetVisible(false)

    -- Clear the old check and set ready state to undecided
    self.readyOverview:ClearOldReadyCheck()

    -- Initialize the new ready check
    self.readyStartTime = Turbine.Engine.GetGameTime()
    self.maxReadyTime = self.readyStartTime + 60
    self.readyActive = true
    --self.readyOverview:MakeNewReadyCheck()

    -- Open acceptance window
    self.readyAcceptanceWindow:MakeReadyCheck()

    -- Start timer
    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end

    -- All done
    Sounds:PlaySoundEffect(SoundEffects["Rc"]) 
end

function ReadyCheckHandler:ToggleButton(state)
    self.ChatReadyButton:SetVisible(state)
end

function ReadyCheckHandler:ReadyUpdate(player, status)
    if self.readyActive then self.readyOverview:UpdateReadycheckMember(player, status) end
end

function ReadyCheckHandler:ReadyCheckActive()
    return self.readyActive
end

function ReadyCheckHandler:ShowOverview()
    self.readyOverview:MakeNewReadyCheck()
    self.readyAcceptanceWindow:SetVisible(false)
end

function ReadyCheckHandler:ReadyDone()
    if self.updateCallback then if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end end
    self.updateCallback = nil
end


ReadyWindow = class(Turbine.UI.Window)
function ReadyWindow:Constructor(parent)
    Turbine.UI.Window.Constructor(self)
    --self:SetSize(32, 32)
    self:SetBackColor(Turbine.UI.Color(0, 0, 0))
    self:SetVisible(false)
    --self:SetPosition(math.floor( Turbine.UI.Display.GetWidth() / 2 - 400), math.floor( Turbine.UI.Display.GetHeight() / 2 - 200))
    self:SetMouseVisible(true)
    self:SetWantsKeyEvents(true)
    --self:SetText("ReadyCheck")
    --self:SetWantsUpdates(false)

    self.KeyDown = function(sender, args)
        if args["Action"] == 145 then
            if self:IsVisible() then self:SetVisible(false) end
        end
    end

    self.HeaderBackground = Turbine.UI.Control()
    self.HeaderBackground:SetSize(500, 20)
    self.HeaderBackground:SetParent(self)
    self.HeaderBackground:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    self.HeaderBackground:SetBackColor(Turbine.UI.Color(0.5, 0.25, 0.25, 0.25))
    self.HeaderBackground:SetMouseVisible(false)

    self.HeaderLabel = Turbine.UI.Label()
    self.HeaderLabel:SetParent(self)
    self.HeaderLabel:SetSize(200, 20)
    self.HeaderLabel:SetPosition(10, 0)
    self.HeaderLabel:SetFont(Turbine.UI.Lotro.Font.TrajanProBold16)
    self.HeaderLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.HeaderLabel:SetText("Prime Readycheck")
    self.HeaderLabel:SetMouseVisible(false)

    self.TimerLabel = Turbine.UI.Label()
    self.TimerLabel:SetPosition(0, 20)
    self.TimerLabel:SetParent(self)
    self.TimerLabel:SetBackColor(Turbine.UI.Color(0, 0.3, 0))
    self.TimerLabel:SetMouseVisible(false)
    self.TimerLabel:SetSize(420, 3)

    self.CloseLabel = Turbine.UI.Label()
    self.CloseLabel:SetParent(self)
    self.CloseLabel:SetSize(20, 20)
    self.CloseLabel:SetBackColor(Turbine.UI.Color(0.3, 0, 0))

    self.CloseLabel.MouseClick = function()
        self:SetVisible(false)
    end

    self.Moving = false

    self.MouseDown = function(sender,args)
        self.MoveX = args.X
        self.MoveY = args.Y
        self.Moving = true
    end
  
    self.MouseUp = function()
        self.Moving = false
    end
  
    self.MouseMove = function(sender,args)
        if self.Moving then
          local newLeft = self:GetLeft() - (self.MoveX - args.X)
          local newTop = self:GetTop() - (self.MoveY - args.Y)
          self:SetPosition(newLeft,newTop)
        end
    end

    self.parent = parent

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.party = self.player:GetParty()
    self.PartyMembers = {}

    -- status - 0 -> undecided | 1 -> not ready | 2 -> ready
    self.PartyStatus = {}
    self.checkSum = 0
    self.maxCheckSum = self.party:GetMemberCount() * 2

    -- Setup window
    for i = 1, self.party:GetMemberCount(), 1 do
        local player = self.party:GetMember(i)
        self.PartyMembers[player:GetName()] = ReadyLabel2(player, self, i)
        self.PartyStatus[player:GetName()] = 0
    end

    self:SetSize(420, 20 * math.ceil(self.party:GetMemberCount() / 4) + 60)
    self:SetPosition(math.floor( Turbine.UI.Display.GetWidth() / 2 - self:GetWidth() / 2), math.floor( Turbine.UI.Display.GetHeight() / 2 - self:GetHeight() / 2))
    self.CloseLabel:SetPosition(self:GetWidth() - 20, 0)

    -- Prepare the window for the new readyCheck
    self.MakeNewReadyCheck = function()
        self:SetVisible(true)
    end
end

function ReadyWindow:ClearOldReadyCheck()
    for key, value in pairs(self.PartyMembers) do
        self.PartyMembers[key]:SetDefault()
        self.PartyStatus[key] = 0
    end
end

function ReadyWindow:SetTimerBar(width)
    self.TimerLabel:SetSize(420 * width, 3)
end

-- We got an update from a partymember handle it
function ReadyWindow:UpdateReadycheckMember(player, state)
    if self.PartyMembers[player] ~= nil and self.PartyStatus[player] < 1 then 
        self.PartyMembers[player]:UpdateReadyState(state)
        if state then
            self.PartyStatus[player] = 2
            self.checkSum = self.checkSum + 2
        else 
            self.PartyStatus[player] = 1 
            self.checkSum = self.checkSum + 1
        end
    end

    if self.checkSum == self.maxCheckSum then
        self:OutputState()
        self.parent:ReadyDone()
    end
end

function ReadyWindow:OutputState()
    if self.checkSum == self.maxCheckSum then
        ChatOutput("<rgb=#00FF00>Everyone is Ready!</rgb>")
    else
        ChatOutput("Readycheck Timer Ended")
        for key, value in pairs(self.PartyStatus) do
            if value == 0 then self:UpdateReadycheckMember(key, false) end
            if value < 2 then ChatOutput("<rgb=#FF0000>" .. key .. "</rgb> is away") end
        end
    end
end

ReadyCheckWindow = class(Turbine.UI.Window)
function ReadyCheckWindow:Constructor()
    Turbine.UI.Lotro.Window.Constructor(self)
    self.activeReadyCheck = false
    self.countdownTimer = 0 --
    local readyCheckStartTime = 0
    local oldGameTime = Turbine.Engine.GetGameTime()
    self.width = 400
    local height = 150
    self:SetSize(self.width,height)
    self:SetPosition( math.floor(Turbine.UI.Display.GetWidth() / 2 - (self.width/2)), math.floor(Turbine.UI.Display.GetHeight() / 2 - (height /2) - 2))
    self:SetWantsKeyEvents(true)
    self:SetVisible(false)
    self:SetZOrder(1)
    --self:SetBackColor(Turbine.UI.Color(0, 0, 0))
    --self:SetWantsUpdates(false)
    self:SetText("Ready Check")
    self:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)    

    self.SetReady = function()
        --self:SetWantsUpdates(false)
        if self.updateCallback then if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end end
        self.updateCallback = nil
        self:SetVisible(false)
    end

    self.timerBarShell = Turbine.UI.Label()
    self.timerBarShell:SetParent(self)
    self.timerBarShell:SetSize(self.width - 20 + 4, 10 + 4)
    --self.timerBarShell:SetBackColor(Turbine.UI.Color(1, 1, 1))
    self.timerBarShell:SetPosition(10 - 2, 80 - 2)

    self.timerBar = Turbine.UI.Label()
    self.timerBar:SetParent(self)
    self.timerBar:SetSize(self.width - 20, 10)
    self.timerBar:SetBackColor(Turbine.UI.Color.DarkGreen)
    self.timerBar:SetPosition(10, 80)

    self.confirmLabel = Turbine.UI.Label()
    self.confirmLabel:SetVisible(true)
    self.confirmLabel:SetFont(Turbine.UI.Lotro.Font.TrajanProBold24)
    self.confirmLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.confirmLabel:SetOutlineColor(Turbine.UI.Color(0,0,0))
    self.confirmLabel:SetText("ARE YOU READY?!")
    self.confirmLabel:SetForeColor(Turbine.UI.Color(1, 1, 1))
    self.confirmLabel:SetParent(self)
    self.confirmLabel:SetSize(400, 40)
    self.confirmLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.confirmLabel:SetPosition(0, 40)

    self.readyButton = Turbine.UI.Lotro.Quickslot()
    self.readyButton:SetParent(self)
    self.readyButton:SetUseOnRightClick(false)
    self.readyButton:SetShortcut()
    self.readyButton:SetPosition(108, 108)
    self.readyButton:SetParent(self)
    self.readyButton:SetMouseVisible((true))
    self.readyButton:SetWantsKeyEvents(true)
    self.readyButton:SetSize(15, 18)

    self.readyCover = Turbine.UI.Label()
    self.readyCover:SetParent(self)
    self.readyCover:SetBackground(RCReady)
    self.readyCover:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.readyCover:SetBlendMode(Turbine.UI.BlendMode.Overlay)
    self.readyCover:SetParent(self)
    self.readyCover:SetSize(24,32)
    self.readyCover:SetPosition(100, 100)
    self.readyCover:SetMouseVisible(false)
    --self.readyCover:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)

    self.ActionCheck = function()
        --self:SetWantsUpdates(false)
        if self.updateCallback then if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end end
        self.updateCallback = nil
        self:SetVisible(false)
    end

    self.notReadyButton = Turbine.UI.Lotro.Quickslot()
    self.notReadyButton:SetParent(self)
    self.notReadyButton:SetUseOnRightClick(false)
    self.notReadyButton:SetShortcut()
    self.notReadyButton:SetPosition(281, 108)
    self.notReadyButton:SetParent(self)
    self.notReadyButton:SetSize(15, 15)

    self.notReadyCover = Turbine.UI.Label()
    self.notReadyCover:SetParent(self)
    self.notReadyCover:SetBackground(RCNotReady)
    self.notReadyCover:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.notReadyCover:SetBlendMode(Turbine.UI.BlendMode.Overlay)
    self.notReadyCover:SetSize(24,32)
    self.notReadyCover:SetPosition(276, 100)
    self.notReadyCover:SetParent(self)
    self.notReadyCover:SetMouseVisible(false)
    --self.notReadyCover:SetBackColorBlendMode(Turbine.UI.BlendMode.None)

    self.notReadyButton.Click = function()
        self.ActionCheck()
    end

    self.MakeReadyCheck = function()
        oldGameTime = Turbine.Engine.GetGameTime()
        self.countdownTimer = 60
        self:SetVisible(true)
        --self:SetWantsUpdates(true)
        if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
        self.timerBar:SetSize(self.width - 20, 10)
        local channel = "f"
        if Party:GetMemberCount() > 6 then
            channel = "ra"
        end
        self.activeReadyCheck = true
        local readyShortcut = Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Alias, "/" .. channel .. " " .. Player:GetName() .. " Ready ")
        local notReadyShortcut = Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Alias, "/" .. channel .. " " .. Player:GetName() .. " Not Ready ")

        self.readyCover.MouseLeave=function(s,a)
            -- hack to prevent real item from being consumed by a click
            local sc = self.readyCover:GetShortcut()
            self.readyCover:SetShortcut(nil)
            self.readyCover:SetShortcut(readyShortcut)
        end	

        self.notReadyCover.MouseLeave=function(s,a)
            -- hack to prevent real item from being consumed by a click
            local sc = self.notReadyCover:GetShortcut()
            self.notReadyCover:SetShortcut(nil)
            self.notReadyCover:SetShortcut(notReadyShortcut)
        end	

        self.readyButton:SetShortcut(readyShortcut)
        self.notReadyButton:SetShortcut(notReadyShortcut)

    end

    self.OngoingReadycheck = function()
        return(self.activeReadyCheck)
    end

    self.ExecuteUpdate = function(delta)
        self.countdownTimer = self.countdownTimer - delta
    
        if self.countdownTimer > 0 then
            self.timerBar:SetSize((self.width - 20) / (60/self.countdownTimer), 10)
        else
            --self:SetWantsUpdates(false)
            if self.updateCallback then if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end end
            self.updateCallback = nil
            self:SetVisible(false)
            self.activeReadyCheck = false
        end
    end
end

ReadyLabel = class(Turbine.UI.Control)
function ReadyLabel:Constructor(player, parent, index)
    Turbine.UI.Control.Constructor(self)
    self:SetParent(parent)
    self:SetVisible(true)
    self:SetSize(80, 300)
    self:SetPosition(80 * index, 20 + 5)
    --self:SetBackColor(Turbine.UI.Color( math.random(), math.random(), math.random() ) )
    self:SetMouseVisible(false)
    --self:SetWantsUpdates(true)

    self.icon = Turbine.UI.Label()
    self.icon:SetParent(self)
    self.icon:SetSize(20, 20)
    self.icon:SetPosition(30, 10)
    self.icon:SetBackground(Icons[player:GetClass()])
    self.icon:SetMouseVisible(false)

    self.label = Turbine.UI.Label()
    self.label:SetParent(self)
    self.label:SetSize(80, 32)
    self.label:SetPosition(0,32)
    self.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.label:SetFont(Turbine.UI.Lotro.Font.BookAntiqua18)
    self.label:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.label:SetMouseVisible(false)
    self.label:SetVisible(true)
    self.label:SetOutlineColor(Turbine.UI.Color(0,0,0))
    --self.label:SetBackColor(Turbine.UI.Color(0, 0, 0))
    self.label:SetText(player:GetName())
    self.label:SetForeColor(Turbine.UI.Color(1, 1, 1))

    self.scrollOffensiveLabel = Turbine.UI.Label()
    self.scrollOffensiveLabel:SetParent(self)
    self.scrollOffensiveLabel:SetSize(80, 32)
    self.scrollOffensiveLabel:SetPosition(0,32 + 32)
    self.scrollOffensiveLabel:SetMouseVisible(false)
    self.scrollOffensiveLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiqua26)
    self.scrollOffensiveLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.scrollOffensiveLabel:SetOutlineColor(OutlineColor)
    self.scrollOffensiveLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)

    self.scrollDefensiveLabel = Turbine.UI.Label()
    self.scrollDefensiveLabel:SetParent(self)
    self.scrollDefensiveLabel:SetSize(80, 32)
    self.scrollDefensiveLabel:SetPosition(0,64 + 32)
    self.scrollDefensiveLabel:SetMouseVisible(false)
    self.scrollDefensiveLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiqua26)
    self.scrollDefensiveLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.scrollDefensiveLabel:SetOutlineColor(OutlineColor)
    self.scrollDefensiveLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)

    self.hopeLabel = Turbine.UI.Label()
    self.hopeLabel:SetParent(self)
    self.hopeLabel:SetSize(80, 32)
    self.hopeLabel:SetPosition(0,96 + 32)
    self.hopeLabel:SetMouseVisible(false)
    self.hopeLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiqua26)
    self.hopeLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.hopeLabel:SetOutlineColor(OutlineColor)
    self.hopeLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)

    self.dbLabel = Turbine.UI.Label()
    self.dbLabel:SetParent(self)
    self.dbLabel:SetSize(80, 32)
    self.dbLabel:SetPosition(0,128 + 32)
    self.dbLabel:SetMouseVisible(false)
    self.dbLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiqua26)
    self.dbLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.dbLabel:SetOutlineColor(OutlineColor)
    self.dbLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)

    local effectList = player:GetEffects()

    local PlayerEffects = {}
    local hope = false
    local oScrolls = false
    local dScrolls = false
    local dB = false
    for i = 1, effectList:GetCount(), 1 do
        PlayerEffects[effectList:Get(i):GetName()] = true
        --ChatOutput(player:GetName() .. effectList:Get(i):GetName())
        if string.find(effectList:Get(i):GetName(), "Battle Lore") then oScrolls = true end
        if string.find(effectList:Get(i):GetName(), "Warding Lore") then dScrolls = true end
        if string.find(effectList:Get(i):GetName(), "Token: Give Hope") then hope = true end
        if string.find(effectList:Get(i):GetName(), "Duty Bound") then dB = true end
    end

    if oScrolls then
        --self.scrollOffensiveLabel:SetBackground(1090642112)
        self.scrollOffensiveLabel:SetText("O")
        self.scrollOffensiveLabel:SetForeColor(Turbine.UI.Color.DarkGreen)
    else
        self.scrollOffensiveLabel:SetText("X")
        self.scrollOffensiveLabel:SetForeColor(Turbine.UI.Color.Crimson)
    end

    if dScrolls then
        self.scrollDefensiveLabel:SetText("O")
        self.scrollDefensiveLabel:SetForeColor(Turbine.UI.Color.DarkGreen)
    else
        self.scrollDefensiveLabel:SetText("X")
        self.scrollDefensiveLabel:SetForeColor(Turbine.UI.Color.Crimson)
    end

    if hope then
        self.hopeLabel:SetText("O")
        self.hopeLabel:SetForeColor(Turbine.UI.Color.DarkGreen)
    else
        self.hopeLabel:SetText("X")
        self.hopeLabel:SetForeColor(Turbine.UI.Color.Crimson)
    end

    if dB then
        self.dbLabel:SetText("O")
        self.dbLabel:SetForeColor(Turbine.UI.Color.DarkGreen)
    else
        self.dbLabel:SetText("X")
        self.dbLabel:SetForeColor(Turbine.UI.Color.Crimson)
    end

    local effectList = player:GetEffects()

    local PlayerEffects = {}
    local hope = false
    local oScrolls = false
    local dScrolls = false
    local dB = false
    for i = 1, effectList:GetCount(), 1 do
        PlayerEffects[effectList:Get(i):GetName()] = true
        --ChatOutput(player:GetName() .. effectList:Get(i):GetName())
        if string.find(effectList:Get(i):GetName(), "Battle Lore") then oScrolls = true end
        if string.find(effectList:Get(i):GetName(), "Warding Lore") then dScrolls = true end
        if string.find(effectList:Get(i):GetName(), "Token: Give Hope") then hope = true end
        if string.find(effectList:Get(i):GetName(), "Duty Bound") then dB = true end
    end

    if oScrolls then
        --self.scrollOffensiveLabel:SetBackground(1090642112)
        self.scrollOffensiveLabel:SetText("O")
        self.scrollOffensiveLabel:SetForeColor(Turbine.UI.Color.DarkGreen)
    end

    if dScrolls then
        self.scrollDefensiveLabel:SetText("O")
        self.scrollDefensiveLabel:SetForeColor(Turbine.UI.Color.DarkGreen)
    end

    if hope then
        self.hopeLabel:SetText("O")
        self.hopeLabel:SetForeColor(Turbine.UI.Color.DarkGreen)
    end

    if dB then
        self.dbLabel:SetText("O")
        self.dbLabel:SetForeColor(Turbine.UI.Color.DarkGreen)
    end

    

    self.ScropeState = function(sender, buffname)
        --ChatOutput(buffname)
        if string.find(buffname,"Battle Lore") then
            --self.scrollOffensiveLabel:SetBackground(1090642112)
            self.scrollOffensiveLabel:SetText("O")
            self.scrollOffensiveLabel:SetForeColor(Turbine.UI.Color.DarkGreen)
        end
    
        if string.find(buffname, "Warding Lore") then
            self.scrollDefensiveLabel:SetText("O")
            self.scrollDefensiveLabel:SetForeColor(Turbine.UI.Color.DarkGreen)
        end
    
        if string.find(buffname, "Token: Give Hope") then
            self.hopeLabel:SetText("O")
            self.hopeLabel:SetForeColor(Turbine.UI.Color.DarkGreen)
        end

        if string.find(buffname, "Duty Bound") then
            self.dbLabel:SetText("O")
            self.dbLabel:SetForeColor(Turbine.UI.Color.DarkGreen)
        end
    end

    self.UpdateReadyState = function(sender, state)
        if state then
            self.label:SetForeColor(Turbine.UI.Color(0, 1, 0))
        else
            self.label:SetForeColor(Turbine.UI.Color(1, 0, 0))
        end
    end

    --ChatOutput(player:GetName() .. "Made Label")

end



--Duty Bound 1090553996
--Dwarf Endurance 1090554033

ReadyButtonWindow = class(Turbine.UI.Window)
function ReadyButtonWindow:Constructor()
    local buttonUpdate = Turbine.Engine.GetGameTime()
    Turbine.UI.Window.Constructor(self)
    self:SetSize(32, 32)
    --self:SetBackColor(Turbine.UI.Color(0, 0, 0))
    self:SetVisible(false)
    self:SetPosition(math.floor( Turbine.UI.Display.GetWidth() / 2 - 16), math.floor( Turbine.UI.Display.GetHeight() / 2 - 16))
    self:SetMouseVisible(true)
    self:SetWantsKeyEvents(true)
    --self:SetWantsUpdates(true)

    self.readyButton = Turbine.UI.Lotro.Quickslot()
    self.readyButton:SetParent(self)
    self.readyButton:SetUseOnRightClick(false)
    self.readyButton:SetShortcut()
    self.readyButton:SetParent(self)
    self.readyButton:SetMouseVisible(true)
    self.readyButton:SetWantsKeyEvents(true)
    self.readyButton:SetSize(32, 32)
    --self.readyButton:SetWantsUpdates(true)

    self.VisibleChanged = function()
        if Party ~= nil then 
            local channel = "f"
            if Party:GetMemberCount() > 6 then
                channel = "ra"
            end
            self.readyButton:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Alias, "/" .. channel .. " " .. " Initiate Readycheck"))
        end
    end
end

ReadyLabel2 = class(Turbine.UI.Control)
function ReadyLabel2:Constructor(player, parent, index)
    Turbine.UI.Control.Constructor(self)
    self:SetParent(parent)
    self:SetVisible(true)
    self:SetSize(100, 20)
    self:SetPosition(15 + 100 * ((index - 1) % 5), 30 + 30 * math.floor(index / 5))
    --self:SetBackColor(Turbine.UI.Color( math.random(), math.random(), math.random() ) )
    self:SetMouseVisible(false)
    --self:SetWantsUpdates(true)

    self.icon = Turbine.UI.Label()
    self.icon:SetParent(self)
    self.icon:SetSize(20, 20)
    self.icon:SetPosition(0, 0)
    self.icon:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend )
    self.icon:SetBackColorBlendMode( Turbine.UI.BlendMode.Multiply )
    self.icon:SetBackground(Icons[player:GetClass()])
    self.icon:SetMouseVisible(false)

    self.label = Turbine.UI.Label()
    self.label:SetParent(self)
    self.label:SetSize(80, 20)
    self.label:SetPosition(20, 0)
    self.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.label:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold18)
    self.label:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.label:SetMouseVisible(false)
    self.label:SetVisible(true)
    self.label:SetMultiline(false)
    self.label:SetOutlineColor(Turbine.UI.Color(0,0,0))
    --self.label:SetBackColor(Turbine.UI.Color(0, 0, 0))
    self.label:SetText(player:GetName())
    self.label:SetForeColor(Turbine.UI.Color(1, 1, 1))

end

function ReadyLabel2:UpdateReadyState(state)
    if state then
        self.label:SetForeColor(Turbine.UI.Color(0, 1, 0))
    else
        self.label:SetForeColor(Turbine.UI.Color(1, 0, 0))
    end
end

function ReadyLabel2:SetDefault()
    self.label:SetForeColor(Turbine.UI.Color(1, 1, 1))
end
