StatEventHandler = class()
function StatEventHandler:Constructor()
    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self:Load()
end

function StatEventHandler:Load()

    self.pluginStartTime = Turbine.Engine.GetGameTime()

    self.charStats = LoadTable(Turbine.DataScope.Character, "PrimeCharStats") or self:GetBaseCharStats()
    
    for key, value in pairs(self:GetBaseCharStats()) do
        if self.charStats[key] == nil then
            self.charStats[key] = value
        end
    end

    self.accountStats = LoadTable(Turbine.DataScope.Account, "PrimeAccountStats") or self:GetBaseAcountStats()
    for key, value in pairs(self:GetBaseAcountStats()) do
        if self.accountStats[key] ==  nil then
            self.accountStats[key] = value
        end
    end
end

function StatEventHandler:Unload()
    local pluginUptime = Turbine.Engine.GetGameTime() - self.pluginStartTime

    -- Daily pvmp stats
    if self.charStats.DailyDeaths > self.charStats["HighestDailyDeaths"] then self.charStats["HighestDailyDeaths"] = self.charStats.DailyDeaths end
    if self.charStats.DailyDeaths > self.accountStats["HighestDailyDeaths"] then self.accountStats["HighestDailyDeaths"] = self.accountStats.DailyDeaths end
    if self.charStats.DailyKB > self.charStats["HighestDailyKBs"] then self.charStats["HighestDailyKBs"] = self.charStats.DailyKB end
    if self.charStats.DailyKB > self.accountStats["HighestDailyKBs"] then self.accountStats["HighestDailyKBs"] = self.accountStats.DailyKB end

    if KillTracker then
        if KillTracker:GetKillStreak() > self.charStats["HighestKillStreak"] then self.charStats["HighestKillStreak"] = KillTracker:GetKillStreak()  end
        if KillTracker:GetKillStreak() > self.accountStats["HighestKillStreak"] then self.accountStats["HighestKillStreak"] = KillTracker:GetKillStreak()  end
        if KillTracker:HighestKillCombo() > self.charStats["HighestKillCombo"] then self.charStats["HighestKillCombo"] = KillTracker:HighestKillCombo() end
        if KillTracker:HighestKillCombo() > self.accountStats["HighestKillCombo"] then self.accountStats["HighestKillCombo"] = KillTracker:HighestKillCombo() end
    end

    self.charStats["Time"] = self.charStats["Time"] + pluginUptime
    self.accountStats["Time"] = self.accountStats["Time"] + pluginUptime

    -- Handle char stats
    SaveTable(Turbine.DataScope.Character, "PrimeCharStats", self.charStats)

    -- Handle account stats
    SaveTable(Turbine.DataScope.Account, "PrimeAccountStats", self.accountStats)
end

function StatEventHandler:GetData()
    return self.charStats, self.accountStats
end

function StatEventHandler:GetAccountData()
    return self.accountStats
end

function StatEventHandler:ResetDailyPvMPStats()
    self.charStats.DailyKB = 0
    self.charStats.DailyDeaths = 0
end

function StatEventHandler:KillStreakDeath(killstreak)
    if killstreak > self.charStats["HighestKillStreak"] then self.charStats["HighestKillStreak"] = killstreak end
    if killstreak > self.accountStats["HighestKillStreak"] then self.accountStats["HighestKillStreak"] = killstreak end
end

function StatEventHandler:AddReload()
    self.charStats["Reload"] = self.charStats["Reload"] + 1
    self.accountStats["Reload"] = self.accountStats["Reload"] + 1
end

function StatEventHandler:AddTrack()
    self.charStats["Tracks"] = self.charStats["Tracks"] + 1
    self.accountStats["Tracks"] = self.accountStats["Tracks"] + 1
end

function StatEventHandler:AddDeath(kb, misadventure)
    if self.charStats["KilledBy"] == nil then  self.charStats["KilledBy"] = {} end
    if self.accountStats["KilledBy"] == nil then  self.accountStats["KilledBy"] = {} end

    self.charStats["Death"] = self.charStats["Death"] + 1
    self.accountStats["Death"] = self.accountStats["Death"] + 1

    if kb == nil and not(misadventure) then return end

    if kb ~= nil then 
        if self.charStats["KilledBy"][kb] == nil then
            self.charStats["KilledBy"][kb] = 1
        else
            self.charStats["KilledBy"][kb] = self.charStats["KilledBy"][kb] + 1 
        end

        if self.accountStats["KilledBy"][kb] == nil then
            self.accountStats["KilledBy"][kb] = 1
        else
            self.accountStats["KilledBy"][kb] = self.accountStats["KilledBy"][kb] + 1 
        end
    elseif misadventure then
        self.charStats["MisadventureDeaths"] = self.charStats["MisadventureDeaths"] + 1
        self.accountStats["MisadventureDeaths"] = self.accountStats["MisadventureDeaths"] + 1
    end

    if KillTracker ~= nil and Settings.EttenmoorsState then 
        self.charStats.DailyDeaths = self.charStats.DailyDeaths + 1
        self.accountStats["PVPDeath"] = self.accountStats["PVPDeath"] + 1
        self.charStats["PVPDeath"] = self.charStats["PVPDeath"] + 1
        KillTracker:UpdateDeath() 

        if Turbine.Gameplay.LocalPlayer.GetInstance():GetAlignment() == 2 then
            self.accountStats["CreepDeaths"] = self.accountStats["CreepDeaths"] + 1
        else
            self.accountStats["FreepDeaths"] = self.accountStats["FreepDeaths"] + 1
        end
    end
end

function StatEventHandler:AddKB(kb, pvp)
    if kb == nil then return end
    if self.charStats["KB"][kb] == nil then
        self.charStats["KB"][kb] = 1
    else
        self.charStats["KB"][kb] = self.charStats["KB"][kb] + 1 
    end

    if self.accountStats["KB"][kb] == nil then
        self.accountStats["KB"][kb] = 1
    else
        self.accountStats["KB"][kb] = self.accountStats["KB"][kb] + 1 
    end

    if pvp and Settings.EttenmoorsState then 
        self.charStats.DailyKB = self.charStats.DailyKB + 1
        if KillTracker ~= nil then KillTracker:UpdateKill(kb) end
        self.accountStats["PVPKB"] = self.accountStats["PVPKB"] + 1
        self.charStats["PVPKB"] = self.charStats["PVPKB"] + 1

        if Turbine.Gameplay.LocalPlayer.GetInstance():GetAlignment() == 2 then
            self.accountStats["CreepKBs"] = self.accountStats["CreepKBs"] + 1
        else
            self.accountStats["FreepKBs"] = self.accountStats["FreepKBs"] + 1
        end
     end 
end

function StatEventHandler:SaveMythicPlusData(instanceID, name, value)
    if not(self.accountStats["MythicPlus"][name]) then self.accountStats["MythicPlus"][name] = {} end
    if not(self.accountStats["MythicPlus"][name][instanceID]) then self.accountStats["MythicPlus"][name][instanceID] = {} end
    table.insert(self.accountStats["MythicPlus"][name][instanceID], value)
end

function StatEventHandler:AddMythicPlusScore(instanceID, name, value)
    if not(self.accountStats["TEQ"][name]) then self.accountStats["TEQ"][name] = {} end
    if not(self.accountStats["TEQ"][name][instanceID]) then 
        self.accountStats["TEQ"][name][instanceID] = value 
    else
        if self.accountStats["TEQ"][name][instanceID]["Score"] < value["Score"] then
            self.accountStats["TEQ"][name][instanceID] = value 
        end
    end
end

function StatEventHandler:ReturnBossData(instanceID, bossName, instanceTier)
    if not(self.accountStats["BossData"][instanceID]) then
        self.accountStats["BossData"][instanceID] = {}
    end

    if not(self.accountStats["BossData"][instanceID][bossName]) then
        self.accountStats["BossData"][instanceID][bossName] = {}
    end

    if not(self.accountStats["BossData"][instanceID][bossName][instanceTier]) then
        self.accountStats["BossData"][instanceID][bossName][instanceTier] = {
            ["Wipes"] = 0,
            ["Completions"] = 0,
            ["BestTime"] = 99999999,
            ["FirstCompletion"] = -1,
        }
    end

    return(self.accountStats["BossData"][instanceID][bossName][instanceTier])
end

function StatEventHandler:SaveBossData(instanceID, bossName, instanceTier, newTime, completion, player)

    -- Update completion/wipe count
    if completion then
        self.accountStats["BossData"][instanceID][bossName][instanceTier]["Completions"] = self.accountStats["BossData"][instanceID][bossName][instanceTier]["Completions"] + 1
    else
        self.accountStats["BossData"][instanceID][bossName][instanceTier]["Wipes"] = self.accountStats["BossData"][instanceID][bossName][instanceTier]["Wipes"] + 1
    end

    -- Update best time
    if self.accountStats["BossData"][instanceID][bossName][instanceTier]["BestTime"] > newTime and completion then
        self.accountStats["BossData"][instanceID][bossName][instanceTier]["BestTime"] = newTime
    end

    -- Update first completion
    if completion and self.accountStats["BossData"][instanceID][bossName][instanceTier]["FirstCompletion"] < 0 then
        self.accountStats["BossData"][instanceID][bossName][instanceTier]["FirstCompletion"] = Turbine.Engine.GetLocalTime()
        self.accountStats["BossData"][instanceID][bossName][instanceTier]["FirstCompletionCount"] = self.accountStats["BossData"][instanceID][bossName][instanceTier]["Wipes"] + 1
        self.accountStats["BossData"][instanceID][bossName][instanceTier]["FirstCompletedCharacter"] = self.player:GetName()

        local group = ""
        if self.player:GetParty() then
            for i = 1, self.player:GetParty():GetMemberCount(), 1 do
                local member = self.player:GetParty():GetMember(i)
                group = group .. member:GetName() .. ", "
            end
        end
        self.accountStats["BossData"][instanceID][bossName][instanceTier]["FirstCompletedGroup"] = group
    end
end

function StatEventHandler:GetBaseAcountStats()
    local result = {}
    result["Time"] = 0
    result["KB"] = {}
    result["Reload"] = 0
    result["Death"] = 0
    result["KilledBy"] = {}
    result["DamageTaken"] = 0
    result["DamageDealt"] = 0
    result["Healing"] = 0
    result["PVPKB"] = 0
    result["PVPDeath"] = 0
    result["MisadventureDeaths"] = 0
    result["HighestDailyDeaths"] = 0
    result["HighestDailyKBs"] = 0
    result["HighestKillStreak"] = 0
    result["HighestKillCombo"] = 0
    result["CreepDeaths"] = 0
    result["CreepKBs"] = 0
    result["FreepDeaths"] = 0
    result["FreepKBs"] = 0
    result["Tracks"] = 0
    result["BossData"] = {}
    result["TEQ"] = {}
    result["Version"] = 0

    return result
end

function StatEventHandler:GetBaseCharStats()
    local result = {}
    result["Time"] = 0
    result["KB"] = {}
    result["Reload"] = 0
    result["Death"] = 0
    result["KilledBy"] = {}
    result["DamageTaken"] = 0
    result["DamageDealt"] = 0
    result["Healing"] = 0
    result["DailyKB"] = 0
    result["DailyDeaths"] = 0
    result["PVPKB"] = 0
    result["PVPDeath"] = 0
    result["MisadventureDeaths"] = 0
    result["HighestDailyDeaths"] = 0
    result["HighestDailyKBs"] = 0
    result["HighestKillStreak"] = 0
    result["HighestKillCombo"] = 0
    result["Tracks"] = 0
    result["Version"] = 0

    return result
end